"""Constants for the Iammeter integration."""
from __future__ import annotations

from collections.abc import Callable
from dataclasses import dataclass
from datetime import datetime

from homeassistant.components.sensor import SensorEntityDescription, SensorStateClass, SensorDeviceClass
from homeassistant.const import (
    ELECTRIC_CURRENT_AMPERE,
    ELECTRIC_POTENTIAL_VOLT,
    ENERGY_KILO_WATT_HOUR,
    FREQUENCY_HERTZ,
    POWER_WATT,
)

DOMAIN = "iammeter_http"

# Default config for iammeter.
DEFAULT_IP = "192.168.2.15"
DEFAULT_PORT = 80
DEFAULT_NAME = "IAMMETER"
DEVICE_3080 = "WEM3080"
DEVICE_3080T = "WEM3080T"
DEVICE_TYPES = [DEVICE_3080, DEVICE_3080T]


@dataclass
class IammeterSensorEntityDescription(SensorEntityDescription):
    """Describes Iammeter sensor entity."""

    value: Callable[[float | int], float] | Callable[[datetime], datetime] | None = None


SENSOR_TYPES_3080: tuple[IammeterSensorEntityDescription, ...] = (
    IammeterSensorEntityDescription(
        key="Voltage",
        name="Voltage",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.VOLTAGE,
    ),
    IammeterSensorEntityDescription(
        key="Current",
        name="Current",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_CURRENT_AMPERE,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.CURRENT,
    ),
    IammeterSensorEntityDescription(
        key="Power",
        name="Power",
        icon="mdi:solar-power",
        native_unit_of_measurement=POWER_WATT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER,
    ),
    IammeterSensorEntityDescription(
        key="ImportEnergy",
        name="ImportEnergy",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="ExportGrid",
        name="ExportGrid",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
)
SENSOR_TYPES_3080T: tuple[IammeterSensorEntityDescription, ...] = (
    IammeterSensorEntityDescription(
        key="Voltage_A",
        name="Voltage_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.VOLTAGE,
    ),
    IammeterSensorEntityDescription(
        key="Current_A",
        name="Current_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_CURRENT_AMPERE,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.CURRENT,
    ),
    IammeterSensorEntityDescription(
        key="Power_A",
        name="Power_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=POWER_WATT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER,
    ),
    IammeterSensorEntityDescription(
        key="ImportEnergy_A",
        name="ImportEnergy_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="ExportGrid_A",
        name="ExportGrid_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="Frequency_A",
        name="Frequency_A",
        icon="mdi:solar-power",
        native_unit_of_measurement=FREQUENCY_HERTZ,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.FREQUENCY,
    ),
    IammeterSensorEntityDescription(
        key="PF_A",
        name="PF_A",
        icon="mdi:solar-power",
        native_unit_of_measurement="",
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER_FACTOR,
    ),
    IammeterSensorEntityDescription(
        key="Voltage_B",
        name="Voltage_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.VOLTAGE,
    ),
    IammeterSensorEntityDescription(
        key="Current_B",
        name="Current_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_CURRENT_AMPERE,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.CURRENT,
    ),
    IammeterSensorEntityDescription(
        key="Power_B",
        name="Power_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=POWER_WATT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER,
    ),
    IammeterSensorEntityDescription(
        key="ImportEnergy_B",
        name="ImportEnergy_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="ExportGrid_B",
        name="ExportGrid_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="Frequency_B",
        name="Frequency_B",
        icon="mdi:solar-power",
        native_unit_of_measurement=FREQUENCY_HERTZ,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.FREQUENCY,
    ),
    IammeterSensorEntityDescription(
        key="PF_B",
        name="PF_B",
        icon="mdi:solar-power",
        native_unit_of_measurement="",
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER_FACTOR,
    ),
    IammeterSensorEntityDescription(
        key="Voltage_C",
        name="Voltage_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.VOLTAGE,
    ),
    IammeterSensorEntityDescription(
        key="Current_C",
        name="Current_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=ELECTRIC_CURRENT_AMPERE,
        state_class=SensorStateClass.MEASUREMENT,
        device_class = SensorDeviceClass.CURRENT,
    ),
    IammeterSensorEntityDescription(
        key="Power_C",
        name="Power_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=POWER_WATT,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER,
    ),
    IammeterSensorEntityDescription(
        key="ImportEnergy_C",
        name="ImportEnergy_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="ExportGrid_C",
        name="ExportGrid_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
        state_class=SensorStateClass.TOTAL_INCREASING,
        device_class=SensorDeviceClass.ENERGY,
    ),
    IammeterSensorEntityDescription(
        key="Frequency_C",
        name="Frequency_C",
        icon="mdi:solar-power",
        native_unit_of_measurement=FREQUENCY_HERTZ,
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.FREQUENCY,
    ),
    IammeterSensorEntityDescription(
        key="PF_C",
        name="PF_C",
        icon="mdi:solar-power",
        native_unit_of_measurement="",
        state_class=SensorStateClass.MEASUREMENT,
        device_class=SensorDeviceClass.POWER_FACTOR,
    ),
)
